/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

// This file contains a number of HTTP and HTML parsing routines
// that come in useful throughput the assignment.

#ifndef _HW4_HTTPUTILS_H_
#define _HW4_HTTPUTILS_H_

#include <stdint.h>

#include <string>
#include <utility>
#include <vector>

using std::pair;
using std::string;
using std::vector;

namespace hw4 {

// This function performs URI decoding.  It scans a string for
// the "%" escape character and converts the token to the
// appropriate ASCII character.  See the wikipedia article on
// URL encoding for an explanation of what's going on here:
//
//    http://en.wikipedia.org/wiki/Percent-encoding
//
string URIDecode(const string &from);

// A URL that's part of a web request has the following structure:
//
//   /foo/bar/baz?field=value;field2=value2
//
//      path     ?   args
//
// This class accepts a URL and splits it into these components and
// URIDecode()'s them, allowing the caller to access the components
// through convenient methods.
class URLParser {
 public:
  explicit URLParser(string url);
  virtual ~URLParser();

  // Return the "path" component of the url, post-uri-decoding.
  string get_path() const;

  // Return the "args" component of the url post-uri-decoding.
  // The args component is parsed into a vector of pairs, where each
  // pair has:
  //
  //   .first = field   .second = value
  vector<pair<string, string> > get_args() const;

 private:
  string url_;
  string path_;
  vector<pair<string, string> > args_;
};

// A convenience routine to manufacture a (blocking) socket to the
// hostname and port number provided as arguments.  Hostname can
// be a DNS name or an IP address, in string form.  On success,
// returns a file descriptor thorugh "client_fd" and returns true.
// On failure, returns false.  Caller is responsible for close()'ing
// the file descriptor.
bool ConnectToServer(string hostname, uint16_t portnum, int *client_fd);

}  // namespace hw4

#endif  // _HW4_HTTPUTILS_H_
