/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <string.h>
#include <iostream>
#include <string>

#include "./HttpConnection.h"
#include "./SocketUtils.h"
#include "./Listener.h"

namespace hw4 {

Listener::Listener(uint16_t portnum, EventLoop *looper,
                   RequestProcessor *reqproc)
  : ss_(portnum), rp_(reqproc) {
  listen_portnum_ = portnum;
  loop_ = looper;

  // Create the listening socket and and register its FD with the
  // event looper.  A new connection arriving makes the listening
  // socket 'readable'.
  std::cout << "  binding/listening to port number... " << portnum;
  std::cout << std::endl;
  assert(ss_.BindAndListen(AF_UNSPEC, &listen_fd_));
  assert(looper->RegisterFD(listen_fd_,
                            EventLoop::FD_IS_READABLE |
                            EventLoop::FD_ERR |
                            EventLoop::FD_HUP,
                            this));
}

Listener::~Listener() {
  // We're being destroyed!  Close the listening socket.
  if (listen_fd_ != 0) {
    close(listen_fd_);
    listen_fd_ = 0;
  }
}

void Listener::Quitting(int fd) { }

void Listener::Handle(const int fd,
                      const unsigned int eventfield,
                      EventLoop *looper) {
  // Is this our listening socket?
  assert(fd == listen_fd_);

  // Make sure this is a "new connection" event.
  assert(eventfield & EventLoop::FD_IS_READABLE);
  assert(!(eventfield & EventLoop::FD_IS_WRITEABLE));
  assert(!(eventfield & EventLoop::FD_ERR));
  assert(!(eventfield & EventLoop::FD_HUP));

  // Accept the incoming request.
  int client_fd;
  uint16_t client_port;
  std::string client_addr, client_dnsname, server_addr, server_dnsname;
  assert(ss_.Accept(&client_fd, &client_addr, &client_port,
                    &client_dnsname, &server_addr, &server_dnsname));

  std::cout << "New connection [" << client_fd << "]:" << std::endl;
  std::cout << "  from " << client_addr;
  std::cout << ":" << client_port << " ";
  std::cout << "[" << client_dnsname << "]" << std::endl;
  std::cout << "  local IP " << server_addr << " ";
  std::cout << "[" << server_dnsname << "]" << std::endl;

  // Create a new HttpConnection object to process the connection.
  HttpConnection *hc;
  hc = new HttpConnection(client_fd, loop_, rp_);
}

}  // namespace hw4
