/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HW4_LISTENER_H_
#define _HW4_LISTENER_H_

#include <stdint.h>
#include <map>

#include "./EventLoop.h"
#include "./EventLoopHandler.h"
#include "./RequestProcessor.h"
#include "./ServerSocket.h"

namespace hw4 {

// A Listener class manages the ServerSocket for the system.  When a
// new connection becomes available, the Listener class invokes
// Accept() on the ServerSocket to establish a new TCP connection with
// the client.  Then, it creates an HttpConnection object to manage
// the connection.
class Listener : public EventLoopHandler {
 public:
  // Constructs a Listener on port "portnum"; registers the listener
  // with "looper" and makes use of the "reqproc" RequestProcessor.
  Listener(uint16_t portnum, EventLoop *looper, RequestProcessor *reqproc);
  virtual ~Listener();

  // This is the EventLoopHandler function that the EventLoop invokes
  // on us when it is shutting down.
  virtual void Quitting(int fd);

  // This is the EventLoopHandler function that the EventLoop invokes
  // on us when a new connection has arrived or when the listener
  // experiences some kind of a socket error.
  virtual void Handle(const int fd,
                      const unsigned int eventfield,
                      EventLoop *looper);

 private:
  // This is the port number we're listening on.
  uint16_t listen_portnum_;

  // This is the descriptor associated with our listening socket.
  int listen_fd_;

  // This is the ServerSocket class managing our listening socket.
  ServerSocket ss_;

  // This is the EventLoop on which we have registered ourselves as a
  // handler for the listening socket.
  EventLoop *loop_;

  // This is the request processor that the HttpConnection objects
  // that we create should use.
  RequestProcessor *rp_;
};  // class Listener

}  // namespace hw4

#endif  // _HW4_LISTENER_H_
