/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HW4_SOCKETLINEREADER_H_
#define _HW4_SOCKETLINEREADER_H_

#include <vector>
#include <string>
#include "./SocketUtils.h"

namespace hw4 {

// A SocketLineReader is a class that reads data from a socket into
// a buffer, splits the data into lines separate by \r\n, and returns
// a list of lines to the caller as they are produced.
class SocketLineReader {
 public:
  explicit SocketLineReader(int fd);
  virtual ~SocketLineReader();

  // This is the the function tht the HttpConnection object invokes
  // when the socket is readable.  The function:
  //
  //  - attempts to read from the socket to append data into its
  //    internal buffer; it reads as much as it can, until the socket
  //    indicates it would block
  //
  //  - as it encounters HTTP line terminators (an HTTP line terminates
  //    with "\r\n"), it appends each line to the output parameter
  //    "linesread", including the "\r\n" at the end of it.
  //
  //  - if the socket is still alive, returns true
  //
  //  - if the socket experiences EOF or an error, returns false
  virtual bool Handle(std::vector<std::string> *linesread);

 private:
  // our file descriptor.
  int fd_;

  // This is the buffer into which we're reading.
  char *buf_;
  unsigned int bufsize_;

  // The current position of the start of the next line, and
  // the offset of the next byte we should read into the buffer.
  unsigned int start_, end_;
};

}  // namespace hw4

#endif  // _HW4_SOCKETLINEREADER_H_
