/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>  // for assert()
#include <unistd.h>  // for read(), write()
#include <errno.h>

#include "./SocketUtils.h"

namespace hw4 {

bool NBWrite(const int fd,
             const char *data,
             const unsigned int bytes_to_write,
             unsigned int *bytes_written) {
  // Defensive programming.
  assert(data != NULL);
  assert(bytes_to_write > 0);

  // Loop on write.
  unsigned int written = 0;
  while (1) {
    int res = write(fd, data+written, (bytes_to_write - written));
    if (res == 0) {
      // Must be EOF.  Return false if we've not written anything yet,
      // true otherwise.
      *bytes_written = written;
      if (written > 0)
        return true;
      return false;
    } else if (res == -1) {
      *bytes_written = written;
      if (errno == EINTR)
        continue;
      if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
        return true;
      return false;
    } else {
      written += res;
      if (written == bytes_to_write) {
        *bytes_written = written;
        return true;
      }
      continue;
    }
  }

  // Should never get here.
  assert(0);
  return false;
}

bool NBRead(const int fd,
            char *data,
            const unsigned int bytes_to_read,
            unsigned int *bytes_read) {
  // Defensive programming.
  assert(data != NULL);
  assert(bytes_to_read > 0);

  // Loop on read.
  unsigned int dread = 0;
  while (1) {
    int res = read(fd, data+dread, (bytes_to_read - dread));
    if (res == 0) {
      // Must be EOF.  Return false if we've not read anything yet,
      // true otherwise.
      *bytes_read = dread;
      if (dread > 0)
        return true;
      return false;
    } else if (res == -1) {
      *bytes_read = dread;
      if (errno == EINTR)
        continue;
      if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
        return true;
      return false;
    } else {
      dread += res;
      if (dread == bytes_to_read) {
        *bytes_read = dread;
        return true;
      }
      continue;
    }
  }

  // Should never get here.
  assert(0);
  return false;
}

}  // namespace hw4

