/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fcntl.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cstdlib>
#include <iostream>
#include <list>
#include <string>

#include "./EventLoop.h"
#include "./RequestProcessor.h"
#include "./Listener.h"

using std::cerr;
using std::cout;
using std::endl;

// Print out program usage, and exit() with EXIT_FAILURE.
void Usage(char *progname);

// Parse the command-line arguments, or invoke Usage() on failure.
list<string> GetPortAndFilenames(int argc, char **argv, uint16_t *port);

int main(int argc, char **argv) {
  if (argc < 3)
    Usage(argv[0]);

  // Print out welcome message.
  cout << "Welcome to http333d, the UW cse333 web server!" << endl;
  cout << "  Copyright 2011 Steven Gribble" << endl;
  cout << "  http://www.cs.washington.edu/homes/gribble" << endl;
  cout << endl;
  cout << "initializing:" << endl;
  cout << "  parsing port number and testing filenames..." << endl;

  // Get the port number and list of index files.
  uint16_t portnum;
  list<string> res = GetPortAndFilenames(argc, argv, &portnum);

  // Create the event loop, listening object, and request processor.
  hw4::EventLoop loop;
  hw4::RequestProcessor *rp;
  rp = new hw4::RequestProcessor(&loop, res);
  hw4::Listener *listen;
  listen = new hw4::Listener(portnum, &loop, rp);

  // Run the main event loop.
  cout << "  initialization complete!" << endl << endl;
  cout << "Processing requests.." << endl;
  loop.Run();

  cout << "completed!  Exiting." << endl;
  return EXIT_SUCCESS;
}

void Usage(char *progname) {
  cerr << "Usage: " << progname << " port index_filenames";
  cerr << endl;
  exit(EXIT_FAILURE);
}

list<string> GetPortAndFilenames(int argc, char **argv, uint16_t *port) {
  // Get the port number.
  uint16_t portnum;
  if (sscanf(argv[1], "%hu", &portnum) != 1)
    Usage(argv[0]);

  // Get the list of files; make sure they exist and are readable.
  list<string> filenames;
  for (int i = 2; i < argc; i++) {
    struct stat fs;
    int fd;
    if (stat(argv[i], &fs) == -1)
      Usage(argv[0]);
    if (!S_ISREG(fs.st_mode)) {
      cerr << argv[i] << " isn't a regular file." << endl;
      Usage(argv[0]);
    }
    fd = open(argv[i], O_RDONLY);
    if (fd == -1) {
      cerr << "Couldn't open " << argv[i] << endl;
      Usage(argv[0]);
    }
    close(fd);

    // Great!  Append this one to our list.
    filenames.push_back(argv[i]);
  }

  // Things worked out, so return the parsed values.
  *port = portnum;
  return filenames;
}

