/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <errno.h>
#include <sys/select.h>
#include <iostream>

#include "./AsyncQuery.h"
#include "./test_suite.h"

#include "./test_asyncquery.h"

namespace hw4 {

TEST_F(Test_AsyncQuery, TestAsyncQueryBasic) {
  list<string> indexfiles;
  indexfiles.push_back(string("./unit_test_indices/bash.idx"));
  indexfiles.push_back(string("./unit_test_indices/books.idx"));
  indexfiles.push_back(string("./unit_test_indices/enron.idx"));
  indexfiles.push_back(string("./unit_test_indices/wiki.idx"));

  int selfpipe_fd;
  AsyncQuery *aq = new AsyncQuery(indexfiles, &selfpipe_fd);
  HW4Addpoints(10);

  // Enqueue a few queries.
  vector<string> q1, q2, q3;
  q1.push_back("ocean");
  q1.push_back("whale");
  q2.push_back("happy");  // test "lots of results" case.
  q3.push_back("supercalifragilisticexpealadocious");  // empty result.
  aq->ProcessQuery(q1, reinterpret_cast<void *>(1));
  aq->ProcessQuery(q2, reinterpret_cast<void *>(2));
  aq->ProcessQuery(q3, reinterpret_cast<void *>(3));

  // Wait for the query results.
  bool got_result[3] = { false, false, false };
  for (int i = 0; i < 3; i++) {
    bool done = false;
    while (!done) {
      char c;

      // Do a select to wait for something to be ready.
      fd_set readset;
      FD_ZERO(&readset);
      FD_SET(selfpipe_fd, &readset);
      int selres = select(selfpipe_fd+1, &readset, NULL, NULL, NULL);
      if (selres == -1) {
        if ((errno == EINTR) || (errno == EAGAIN))
          continue;
        FAIL() << "select failed!";
        done = true;
        continue;
      }
      ASSERT_EQ(1, selres);
      ASSERT_NE(0, FD_ISSET(selfpipe_fd, &readset));

      ssize_t res = read(selfpipe_fd, &c, 1);
      if (res == 1) {
        // Got a result, pick up from the queue.
        // Test it to see which result it is, set done
        // if all gotten.
        done = true;
        AsyncQuery::QueryTask *t = aq->DequeueResults();
        ASSERT_NE(reinterpret_cast<void *>(NULL), t);
        if (((uint64_t) t->v) == 1) {
          if (got_result[0]) {
            FAIL() << "got_result[0] was already true!";
          } else {
            got_result[0] = true;
          }
        } else if (((uint64_t) t->v) == 2) {
          if (got_result[1]) {
            FAIL() << "got_result[1] was already true!";
          } else {
            got_result[1] = true;
          }
        } else if (((uint64_t) t->v) == 3) {
          if (got_result[2]) {
            FAIL() << "got_result[2] was already true!";
          } else {
            got_result[2] = true;
          }
        } else {
          FAIL() << "t->v wasn't 1, 2, or 3";
        }
        delete t;
      } else if (res == 0) {
        // bummer!  eof shoudn't happen.
        FAIL() << "hit EOF on the readpipe";
        done = true;
      } else if (res == -1) {
        if (!((errno == EAGAIN) || (errno == EINTR))) {
          FAIL() << "hit a non-EAGAIN/EINTR failure on the read pipe.";
          done = true;
        }
      }
    }
  }
  ASSERT_TRUE(got_result[0]);
  ASSERT_TRUE(got_result[1]);
  ASSERT_TRUE(got_result[2]);
  delete aq;

  HW4Addpoints(20);
}

}  // namespace hw4
