/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdint.h>
#include <iostream>
#include <sstream>
#include <string>

#include "./test_eventloop.h"
#include "./EventLoop.h"

namespace hw4 {

static unsigned int elh_count = 0;

// This class is what the eventloop unit test uses to test out
// event handling.
class TestELH : public EventLoopHandler {
 public:
  TestELH(bool *fd_handled, int *handlecount, EventLoop *looper) {
    handlecount_ = handlecount;
    fd_handled_ = fd_handled;
    loop_ = looper;
    expected_count_ = 0;
    fd_ = -1;
    fd_count_ = 0;

    if (elh_count == 0) {
      expected_count_ = 14795008;
      fd_ = open("./unit_test_indices/bash.idx", O_RDONLY | O_NONBLOCK);
      elh_count++;
    } else if (elh_count == 1) {
      expected_count_ = 5317498;
      fd_ = open("./unit_test_indices/enron.idx", O_RDONLY | O_NONBLOCK);
      elh_count++;
    } else {
      expected_count_ = 1235142;
      fd_ = open("./test_tree/books/mobydick.txt", O_RDONLY | O_NONBLOCK);
      elh_count++;
    }

    if (fd_ == -1) {
      std::cerr << "Failed to open a FD in TestEventLoopBasic.";
      std::cerr << std::endl;
      assert(0);
    }

    // Register ourselves with the event loop.
    assert(looper->RegisterFD(fd_,
                              EventLoop::FD_IS_READABLE |
                              EventLoop::FD_ERR |
                              EventLoop::FD_HUP,
                              this));
  }

  ~TestELH() {
    // The event loop deletes us when we signal we're done.
    // Close the file descriptor.
    close(fd_);
    elh_count--;
    if (elh_count == 0) {
      loop_->Quit();
    }
  }

  // This is our "quitting" method.
  void Quitting(int fd) { }

  // This is our event handler method.
  void Handle(const int fd,
              const unsigned int eventfield,
              EventLoop *looper) {
    unsigned char buf[1024] = {0};  // Buffer for reading/writing.
    ssize_t rwcount;

    (*handlecount_)++;

    assert((eventfield & EventLoop::FD_IS_READABLE) != (unsigned int) 0);
    assert((eventfield & EventLoop::FD_IS_WRITEABLE) == (unsigned int) 0);
    assert((eventfield & EventLoop::FD_ERR) == (unsigned int) 0);
    assert((eventfield & EventLoop::FD_HUP) == (unsigned int) 0);
    rwcount = read(fd, &(buf[0]), 1024);
    if (rwcount > 0) {
      // Data was read, update the read count.
      fd_count_ += rwcount;
    } else if (rwcount == 0) {
      // Hit the end of file!
      assert(expected_count_ == fd_count_);
      assert(looper->UnregisterFD(fd));
      assert(!looper->UnregisterFD(fd));
      *fd_handled_ = true;
    } else  if (rwcount == -1) {
      if ((errno == EAGAIN) || (errno == EINTR)) {
        // need to try again later.
      } else {
        // An error case?!
        std::cerr << "read on fd return a failure: " << errno;
        std::cerr << std::endl;
        assert(0);
      }
    }
  }

 private:
  std::string fname_;
  unsigned int expected_count_;
  int fd_;
  int *handlecount_;
  bool *fd_handled_;
  unsigned int fd_count_;
  EventLoop *loop_;
};

// Test our ability to exercise the event loop and handler.
TEST_F(Test_EventLoop, TestEventLoopBasic) {
  // Create our event loop.
  EventLoop looper;

  bool fd1_handled = false;
  bool fd2_handled = false;
  bool fd3_handled = false;
  int handlecount = 0;

  // Create our TestELH handler.
  TestELH *newELH;
  newELH = new TestELH(&fd1_handled, &handlecount, &looper);
  newELH = new TestELH(&fd2_handled, &handlecount, &looper);
  newELH = new TestELH(&fd3_handled, &handlecount, &looper);

  // Run the event loop.
  looper.Run();

  // Make sure all three event handlers were invoked.
  EXPECT_EQ(true, fd1_handled);
  EXPECT_EQ(true, fd2_handled);
  EXPECT_EQ(true, fd3_handled);
  EXPECT_GT(handlecount, 0);
}

}  // namespace hw4
