/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string>

#include "./test_suite.h"
#include "./HttpUtils.h"

#include "./test_httputils.h"

using std::string;

namespace hw4 {

TEST_F(Test_HttpUtils, TestHttpUtilsBasic) {
  // Test out URIDecoding.
  string empty("");
  string plain("foo");
  string two("%74%77%6f");
  string twoupper("%74%77%6F");
  string nope("%16nope");
  string broken("%broken%1");
  string spacey("%20+blah blah");
  ASSERT_EQ(string(""), URIDecode(empty));
  ASSERT_EQ(string("foo"), URIDecode(plain));
  ASSERT_EQ(string("two"), URIDecode(two));
  ASSERT_EQ(string("two"), URIDecode(twoupper));
  ASSERT_EQ(string("%16nope"), URIDecode(nope));
  ASSERT_EQ(string("%broken%1"), URIDecode(broken));
  ASSERT_EQ(string("  blah blah"), URIDecode(spacey));

  // Test out URL parsing.
  string easy("/foo/bar");
  string tricky("/foo/bar?");
  string query("/foo/bar?foo=blah+blah");
  string many("/foo/bar?foo=bar;bam=baz;");
  string manyshort("/foo/bar?foo=bar;bam=baz");

  URLParser p(easy);
  ASSERT_EQ("/foo/bar", p.get_path());
  ASSERT_EQ((unsigned) 0, p.get_args().size());

  p = URLParser(tricky);
  ASSERT_EQ("/foo/bar", p.get_path());
  ASSERT_EQ((unsigned) 0, p.get_args().size());

  p = URLParser(query);
  ASSERT_EQ("/foo/bar", p.get_path());
  ASSERT_EQ((unsigned) 1, p.get_args().size());
  ASSERT_EQ("foo", p.get_args()[0].first);
  ASSERT_EQ("blah blah", p.get_args()[0].second);

  p = URLParser(many);
  ASSERT_EQ("/foo/bar", p.get_path());
  ASSERT_EQ((unsigned) 2, p.get_args().size());
  ASSERT_EQ("foo", p.get_args()[0].first);
  ASSERT_EQ("bar", p.get_args()[0].second);
  ASSERT_EQ("bam", p.get_args()[1].first);
  ASSERT_EQ("baz", p.get_args()[1].second);

  p = URLParser(manyshort);
  ASSERT_EQ("/foo/bar", p.get_path());
  ASSERT_EQ((unsigned) 2, p.get_args().size());
  ASSERT_EQ("foo", p.get_args()[0].first);
  ASSERT_EQ("bar", p.get_args()[0].second);
  ASSERT_EQ("bam", p.get_args()[1].first);
  ASSERT_EQ("baz", p.get_args()[1].second);

  HW4Addpoints(20);
}

}  // namespace hw4
