/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream>

#include "./test_suite.h"
#include "./EventLoop.h"
#include "./HttpUtils.h"
#include "./Listener.h"
#include "./RequestProcessor.h"
#include "./test_listener.h"

namespace hw4 {

unsigned short GetRandPort();

static const char *request_string =
  "GET / HTTP/1.1\r\n"
  "Host: localhost:%hu\r\n"
  "User-Agent: UnitTest\r\n"
  "Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*\r\n"
  "Accept-Language: en-us,en;q=0.5\r\n"
  "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7\r\n"
  "Keep-Alive: 115\r\n"
  "Connection: keep-alive\r\n\r\n";

struct ThreadArg {
  pthread_mutex_t  count_mutex;
  uint32_t         connection_count;
  uint16_t         serverport;
  EventLoop        *loop;
};

void *ClientThread(void *v) {
  ThreadArg *arg = reinterpret_cast<ThreadArg *>(v);

  // Connect to the server.
  int clientsock;
  assert(ConnectToServer("localhost", arg->serverport, &clientsock));

  // Write an http request.  Note that the listener might shut down
  // early, so it's not an error if the write fails.
  char req[strlen(request_string)+10];
  snprintf(req,
           strlen(request_string)+10,
           request_string,
           arg->serverport);
  unsigned int writelen = write(clientsock, req, strlen(req));
  assert(writelen == strlen(req));

  // Tryto get (some of) the reply; note the listener might shut
  // down early, so it's not an error if the read fails.
  char resp[101] = { 0 };
  unsigned int readlen = read(clientsock, resp, 100);
  close(clientsock);
  assert(readlen > 0);
  resp[readlen] = '\0';

  // Increment the count.
  assert(pthread_mutex_lock(&(arg->count_mutex)) == 0);
  arg->connection_count++;
  if (arg->connection_count == 3) {
    std::cout << "Quitting" << std::endl;
    arg->loop->Quit();
    // Do a quick connection to tickle the event loop into quitting.
    if (ConnectToServer("localhost", arg->serverport, &clientsock)) {
      close(clientsock);
    }
  }
  assert(pthread_mutex_unlock(&(arg->count_mutex)) == 0);

  // Exit this thread.
  return NULL;
}

TEST_F(Test_Listener, TestListenerBasic) {
  // Create the event looper.
  EventLoop loop;

  // Prepare a request processor.
  std::list<std::string> idxlist;
  idxlist.push_back("./unit_test_indices/enron.idx");
  RequestProcessor *rp = new RequestProcessor(&loop, idxlist);

  unsigned short portnum = GetRandPort();
  std::cout << "Starting listener on port " << portnum << std::endl;
  Listener *lstr;
  lstr = new Listener(portnum, &loop, rp);

  // Spawn three threads.
  struct ThreadArg ta;
  pthread_t threads[3];
  assert(pthread_mutex_init(&ta.count_mutex, NULL) == 0);
  ta.connection_count = 0;
  ta.serverport = portnum;
  ta.loop = &loop;

  for (int i = 0; i < 3; i++) {
    assert(pthread_create(&threads[i], NULL, ClientThread, &ta) == 0);
  }

  // Run the event loop.
  loop.Run();

  // Join with the client threads.
  for (int i = 0; i < 3; i++) {
    assert(pthread_join(threads[i], NULL) == 0);
  }

  ASSERT_EQ((uint32_t) 3, ta.connection_count);
  HW4Addpoints(30);
}

}  // namespace hw4
