/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <boost/scoped_ptr.hpp>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>

#include <string>
#include <iostream>

#include "./test_suite.h"
#include "./HttpUtils.h"
#include "./ServerSocket.h"

#include "./test_serversocket.h"

namespace hw4 {

unsigned short GetRandPort();

TEST_F(Test_ServerSocket, TestServerSocketBasic) {
  {
    // Create the server listening socket.
    int listen_fd;
    unsigned short portnum = GetRandPort();

    boost::scoped_ptr<ServerSocket> ss(new ServerSocket(portnum));
    EXPECT_TRUE(ss->BindAndListen(AF_INET6, &listen_fd));

    // Connect to it as a client.
    int clientsock;
    assert(ConnectToServer("localhost", portnum, &clientsock));

    // Accept the connection as a server.
    int client_fd;
    std::string caddr, cdns, saddr, sdns;
    uint16_t cport;

    ASSERT_TRUE(ss->Accept(&client_fd, &caddr, &cport,
                           &cdns, &saddr, &sdns));

    // Try to accept a second connection; should return false.
    int client_fd2;
    ASSERT_FALSE(ss->Accept(&client_fd2, &caddr, &cport,
                            &cdns, &saddr, &sdns));

    // Write some data to the server as the client.
    const char *hi = "Hello!\n";
    char recvbuf[8] = { 0 };
    int res;
    while (1) {
      res = write(clientsock, hi, 7);
      if (res != 7) {
        assert(res < 0);
        if (res == -1) {
          if ((errno == EAGAIN) || (errno == EINTR))
            continue;
          std::cerr << strerror(errno) << std::endl;
          assert(0);
        }
      } else {
        break;
      }
    }
    // Read some data from the server as the client.
    ASSERT_EQ(7, read(client_fd, recvbuf, 7));
    ASSERT_EQ(0, strcmp(recvbuf, hi));

    // Done! Close up shop.
    close(client_fd);
    close(clientsock);
  }
  HW4Addpoints(20);
}

}  // namespace hw4
