/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <boost/scoped_ptr.hpp>
#include <string>
#include <vector>

#include "./test_suite.h"
#include "./HttpUtils.h"
#include "./ServerSocket.h"
#include "./SocketLineReader.h"
#include "./SocketUtils.h"
#include "./test_socketlinereader.h"

namespace hw4 {

unsigned short GetRandPort();

TEST_F(Test_SocketLineReader, TestSocketLineReaderBasic) {
  // Create a server listening socket.
  int listen_fd;
  unsigned short portnum = GetRandPort();

  boost::scoped_ptr<ServerSocket> ss(new ServerSocket(portnum));
  ASSERT_TRUE(ss->BindAndListen(AF_UNSPEC, &listen_fd));

  // Connect to it as a client.
  int clientsock;
  assert(ConnectToServer("localhost", portnum, &clientsock));

  // Accept the connection as a server.
  int client_fd;
  std::string caddr, cdns, saddr, sdns;
  uint16_t cport;

  ASSERT_TRUE(ss->Accept(&client_fd, &caddr, &cport,
                         &cdns, &saddr, &sdns));

  // Write a few lines to the server.
  char *bigstr = new char[2048];
  unsigned int written;
  for (int i=0; i<2046; i++)
    bigstr[i] = 'H';
  bigstr[2046] = '\r';
  bigstr[2047] = '\n';

  assert(NBWrite(clientsock, bigstr, 2048, &written));
  assert(written == 2048);
  assert(NBWrite(clientsock, "there\r\n\r\nyo",
                 strlen("there\r\n\r\nyo"), &written));
  assert(written == strlen("there\r\n\r\nyo"));
  assert(NBWrite(clientsock, "yoma\r\n",
                 strlen("yoma\r\n"), &written));
  assert(written == strlen("yoma\r\n"));
  delete[] bigstr;
  close(clientsock);

  // Create the SocketLineReader.
  SocketLineReader slr(client_fd);

  // Keep looping on handle until we're done.
  std::vector<std::string> lines;
  while(slr.Handle(&lines));
  ASSERT_EQ((unsigned) 4, (lines.size()));
  ASSERT_EQ((unsigned) 2048, lines[0].size());
  ASSERT_EQ(std::string("there\r\n"), lines[1]);
  ASSERT_EQ(std::string("\r\n"), lines[2]);
  ASSERT_EQ(std::string("yoyoma\r\n"), lines[3]);

  ASSERT_FALSE(slr.Handle(&lines));

  HW4Addpoints(20);
}

}  // namespace hw4
