/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <boost/scoped_ptr.hpp>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>

#include "./test_suite.h"
#include "./ServerSocket.h"
#include "./SocketUtils.h"

#include "./test_socketutils.h"

namespace hw4 {

unsigned short GetRandPort() {
  unsigned short portnum = 10000;
  portnum += ((unsigned short) getpid()) % 25000;
  portnum += ((unsigned short) rand()) % 5000;
  return portnum;
}

TEST_F(Test_SocketUtils, TestSocketUtilsBasic) {
  // Open up a server socket and a client socket.
  int listen_fd;
  unsigned short portnum = GetRandPort();

  boost::scoped_ptr<ServerSocket> ss(new ServerSocket(portnum));
  ASSERT_TRUE(ss->BindAndListen(AF_UNSPEC, &listen_fd));

  // Connect to it as a client.
  int clientsock;
  struct hostent *host;
  struct sockaddr_in server_addr;
  host = gethostbyname("localhost");
  if ((clientsock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    std::cerr << strerror(errno) << std::endl;
    assert(0);
  }
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(portnum);
  server_addr.sin_addr = *((struct in_addr *) host->h_addr);
  bzero(&(server_addr.sin_zero), 8);
  if (connect(clientsock, (struct sockaddr *) &server_addr,
              sizeof(struct sockaddr)) == -1) {
    std::cerr << strerror(errno) << std::endl;
    assert(0);
  }

  // Accept the connection as a server.
  int client_fd;
  std::string caddr, cdns, saddr, sdns;
  uint16_t cport;

  ASSERT_TRUE(ss->Accept(&client_fd, &caddr, &cport,
                         &cdns, &saddr, &sdns));
  HW4Addpoints(10);

  // Write some data to the server as the client.
  const char *hi = "Hello!\n";
  unsigned int num_written = 0, total_written = 0;
  while (1) {
    ASSERT_TRUE(NBWrite(clientsock,
                        hi+total_written,
                        7-total_written,
                        &num_written));
    total_written += num_written;
    if (total_written == 7)
      break;
  }

  // Read some data as the server.
  char recvbuf[10];
  unsigned int num_read = 0, total_read = 0;
  while (1) {
    ASSERT_TRUE(NBRead(client_fd,
                       recvbuf+total_read,
                       7-total_read,
                       &num_read));
    total_read += num_read;
    if (total_read == 7)
      break;
  }

  // Close the client socket.
  close(clientsock);

  // Make sure reading from the server-side fails.
  ASSERT_FALSE(NBRead(client_fd, recvbuf, 1, &num_read));
  close(client_fd);

  HW4Addpoints(20);
}

}  // namespace hw4
