/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>

#include "./ThreadPool.h"
#include "./test_suite.h"

#include "./test_threadpool.h"

namespace hw4 {

uint32_t threadcount = 0;
uint32_t workcount = 0;
static pthread_mutex_t mtx;

void TestTaskFn(ThreadPool::Task *t) {
  assert(pthread_mutex_lock(&mtx) == 0);
  workcount++;
  assert(pthread_mutex_unlock(&mtx) == 0);

  if (workcount % 5 == 1) {
    usleep(250000);  // 0.25s
  }
  delete t;
}

void *TestInitFn(void *arg) {
  // Do very little in the per-thread init function call.
  EXPECT_EQ(reinterpret_cast<void *>(0xDEADBEEF), arg);
  threadcount++;
  return reinterpret_cast<void *>(threadcount);
}

TEST_F(Test_ThreadPool, TestThreadPoolBasic) {
  assert(pthread_mutex_init(&mtx, NULL) == 0);
  ThreadPool *tp = new ThreadPool(10,
                                  TestInitFn,
                                  reinterpret_cast<void *>(0xDEADBEEF));

  // Try dispatching some work.  Make sure we dispatch more than will
  // finish, so that the destructor case with active work threads
  // running and things in the queue is exercised fully.
  for (int i = 0; i < 300; i++) {
    ThreadPool::Task *next_t = new ThreadPool::Task(TestTaskFn);

    tp->Dispatch(next_t);
  }
  usleep(1250000);  // 1.25s
  delete tp;
}

}  // namespace hw4
